# [server]
# db_user = root
# db_password = password
# db_database = database_name
# db_host = 127.0.0.1/ you best not add into git
# workers_num = 10
# [test]
# db_user = root
# db_password = password
# db_database = database_name
# db_host = 127.0.0.1
# workers_num = 10
